@@ -49,7 +49,6 @@ pub enum IoKind {
     StdErr,
 }
 
-type Requests = Arc<Mutex<HashMap<u64, oneshot::Sender<Result<Response>>>>>;
 type LogHandlers = Arc<Mutex<SmallVec<[(LogKind, IoHandler); 2]>>>;
 
 pub trait Transport: Send + Sync {
@@ -63,7 +62,7 @@ pub trait Transport: Send + Sync {
             Box<dyn AsyncRead + Unpin + Send + 'static>,
         )>,
     >;
-    fn kill(&self);
+    fn kill(&mut self);
     #[cfg(any(test, feature = "test-support"))]
     fn as_fake(&self) -> &FakeTransport {
         unreachable!()
@@ -93,9 +92,11 @@ async fn start(
 
 pub(crate) struct TransportDelegate {
     log_handlers: LogHandlers,
-    pending_requests: Requests,
+    // TODO this should really be some kind of associative channel
+    pub(crate) pending_requests:
+        Arc<Mutex<Option<HashMap<u64, oneshot::Sender<Result<Response>>>>>>,
     pub(crate) transport: Mutex<Box<dyn Transport>>,
-    server_tx: smol::lock::Mutex<Option<Sender<Message>>>,
+    pub(crate) server_tx: smol::lock::Mutex<Option<Sender<Message>>>,
     tasks: Mutex<Vec<Task<()>>>,
 }
 
@@ -107,7 +108,7 @@ impl TransportDelegate {
             transport: Mutex::new(transport),
             log_handlers,
             server_tx: Default::default(),
-            pending_requests: Default::default(),
+            pending_requests: Arc::new(Mutex::new(Some(HashMap::default()))),
             tasks: Default::default(),
         })
     }
@@ -148,16 +149,26 @@ impl TransportDelegate {
                 .await
                 {
                     Ok(()) => {
-                        pending_requests.lock().drain().for_each(|(_, request)| {
-                            request
-                                .send(Err(anyhow!("debugger shutdown unexpectedly")))
-                                .ok();
-                        });
+                        pending_requests
+                            .lock()
+                            .take()
+                            .into_iter()
+                            .flatten()
+                            .for_each(|(_, request)| {
+                                request
+                                    .send(Err(anyhow!("debugger shutdown unexpectedly")))
+                                    .ok();
+                            });
                     }
                     Err(e) => {
-                        pending_requests.lock().drain().for_each(|(_, request)| {
-                            request.send(Err(e.cloned())).ok();
-                        });
+                        pending_requests
+                            .lock()
+                            .take()
+                            .into_iter()
+                            .flatten()
+                            .for_each(|(_, request)| {
+                                request.send(Err(e.cloned())).ok();
+                            });
                     }
                 }
             }));
@@ -182,15 +193,6 @@ impl TransportDelegate {
         self.transport.lock().tcp_arguments()
     }
 
-    pub(crate) fn add_pending_request(
-        &self,
-        sequence_id: u64,
-        request: oneshot::Sender<Result<Response>>,
-    ) {
-        let mut pending_requests = self.pending_requests.lock();
-        pending_requests.insert(sequence_id, request);
-    }
-
     pub(crate) async fn send_message(&self, message: Message) -> Result<()> {
         if let Some(server_tx) = self.server_tx.lock().await.as_ref() {
             server_tx.send(message).await.context("sending message")
@@ -284,7 +286,7 @@ impl TransportDelegate {
     async fn recv_from_server<Stdout>(
         server_stdout: Stdout,
         mut message_handler: DapMessageHandler,
-        pending_requests: Requests,
+        pending_requests: Arc<Mutex<Option<HashMap<u64, oneshot::Sender<Result<Response>>>>>>,
         log_handlers: Option<LogHandlers>,
     ) -> Result<()>
     where
@@ -294,16 +296,21 @@ impl TransportDelegate {
         let mut reader = BufReader::new(server_stdout);
 
         let result = loop {
-            match Self::receive_server_message(&mut reader, &mut recv_buffer, log_handlers.as_ref())
-                .await
-            {
+            let result =
+                Self::receive_server_message(&mut reader, &mut recv_buffer, log_handlers.as_ref())
+                    .await;
+            match result {
                 ConnectionResult::Timeout => anyhow::bail!("Timed out when connecting to debugger"),
                 ConnectionResult::ConnectionReset => {
                     log::info!("Debugger closed the connection");
-                    return Ok(());
+                    break Ok(());
                 }
                 ConnectionResult::Result(Ok(Message::Response(res))) => {
-                    let tx = pending_requests.lock().remove(&res.request_seq);
+                    let tx = pending_requests
+                        .lock()
+                        .as_mut()
+                        .context("client is closed")?
+                        .remove(&res.request_seq);
                     if let Some(tx) = tx {
                         if let Err(e) = tx.send(Self::process_response(res)) {
                             log::trace!("Did not send response `{:?}` for a cancelled", e);
@@ -354,7 +361,6 @@ impl TransportDelegate {
         let mut content_length = None;
         loop {
             buffer.truncate(0);
-
             match reader.read_line(buffer).await {
                 Ok(0) => return ConnectionResult::ConnectionReset,
                 Ok(_) => {}
@@ -412,21 +418,6 @@ impl TransportDelegate {
         ConnectionResult::Result(message)
     }
 
-    pub async fn shutdown(&self) -> Result<()> {
-        log::debug!("Start shutdown client");
-
-        if let Some(server_tx) = self.server_tx.lock().await.take().as_ref() {
-            server_tx.close();
-        }
-
-        self.pending_requests.lock().clear();
-        self.transport.lock().kill();
-
-        log::debug!("Shutdown client completed");
-
-        anyhow::Ok(())
-    }
-
     pub fn has_adapter_logs(&self) -> bool {
         self.transport.lock().has_adapter_logs()
     }
@@ -546,7 +537,7 @@ impl Transport for TcpTransport {
         true
     }
 
-    fn kill(&self) {
+    fn kill(&mut self) {
         if let Some(process) = &mut *self.process.lock() {
             process.kill();
         }
@@ -613,13 +604,13 @@ impl Transport for TcpTransport {
 impl Drop for TcpTransport {
     fn drop(&mut self) {
         if let Some(mut p) = self.process.lock().take() {
-            p.kill();
+            p.kill()
         }
     }
 }
 
 pub struct StdioTransport {
-    process: Mutex<Child>,
+    process: Mutex<Option<Child>>,
     _stderr_task: Option<Task<()>>,
 }
 
@@ -660,7 +651,7 @@ impl StdioTransport {
             ))
         });
 
-        let process = Mutex::new(process);
+        let process = Mutex::new(Some(process));
 
         Ok(Self {
             process,
@@ -674,8 +665,10 @@ impl Transport for StdioTransport {
         false
     }
 
-    fn kill(&self) {
-        self.process.lock().kill()
+    fn kill(&mut self) {
+        if let Some(process) = &mut *self.process.lock() {
+            process.kill();
+        }
     }
 
     fn connect(
@@ -686,8 +679,9 @@ impl Transport for StdioTransport {
             Box<dyn AsyncRead + Unpin + Send + 'static>,
         )>,
     > {
-        let mut process = self.process.lock();
         let result = util::maybe!({
+            let mut guard = self.process.lock();
+            let process = guard.as_mut().context("oops")?;
             Ok((
                 Box::new(process.stdin.take().context("Cannot reconnect")?) as _,
                 Box::new(process.stdout.take().context("Cannot reconnect")?) as _,
@@ -703,7 +697,9 @@ impl Transport for StdioTransport {
 
 impl Drop for StdioTransport {
     fn drop(&mut self) {
-        self.process.get_mut().kill();
+        if let Some(process) = &mut *self.process.lock() {
+            process.kill();
+        }
     }
 }
 
@@ -723,6 +719,7 @@ pub struct FakeTransport {
 
     stdin_writer: Option<PipeWriter>,
     stdout_reader: Option<PipeReader>,
+    message_handler: Option<Task<Result<()>>>,
 }
 
 #[cfg(any(test, feature = "test-support"))]
@@ -774,18 +771,19 @@ impl FakeTransport {
         let (stdin_writer, stdin_reader) = async_pipe::pipe();
         let (stdout_writer, stdout_reader) = async_pipe::pipe();
 
-        let this = Self {
+        let mut this = Self {
             request_handlers: Arc::new(Mutex::new(HashMap::default())),
             response_handlers: Arc::new(Mutex::new(HashMap::default())),
             stdin_writer: Some(stdin_writer),
             stdout_reader: Some(stdout_reader),
+            message_handler: None,
         };
 
         let request_handlers = this.request_handlers.clone();
         let response_handlers = this.response_handlers.clone();
         let stdout_writer = Arc::new(smol::lock::Mutex::new(stdout_writer));
 
-        cx.background_spawn(async move {
+        this.message_handler = Some(cx.background_spawn(async move {
             let mut reader = BufReader::new(stdin_reader);
             let mut buffer = String::new();
 
@@ -833,7 +831,6 @@ impl FakeTransport {
                                             .unwrap();
 
                                     let mut writer = stdout_writer.lock().await;
-
                                     writer
                                         .write_all(
                                             TransportDelegate::build_rpc_message(message)
@@ -870,8 +867,7 @@ impl FakeTransport {
                     }
                 }
             }
-        })
-        .detach();
+        }));
 
         Ok(this)
     }
@@ -904,7 +900,9 @@ impl Transport for FakeTransport {
         false
     }
 
-    fn kill(&self) {}
+    fn kill(&mut self) {
+        self.message_handler.take();
+    }
 
     #[cfg(any(test, feature = "test-support"))]
     fn as_fake(&self) -> &FakeTransport {
