@@ -23,12 +23,14 @@ pub enum IconName {
     AiZed,
     ArrowCircle,
     ArrowDown,
+    ArrowDown10,
     ArrowDownFromLine,
     ArrowDownRight,
     ArrowLeft,
     ArrowRight,
     ArrowRightLeft,
     ArrowUp,
+    ArrowUpAlt,
     ArrowUpFromLine,
     ArrowUpRight,
     ArrowUpRightAlt,
@@ -44,6 +46,7 @@ pub enum IconName {
     Blocks,
     Bolt,
     BoltFilled,
+    BoltFilledAlt,
     Book,
     BookCopy,
     BookPlus,
@@ -162,6 +165,9 @@ pub enum IconName {
     ListX,
     LoadCircle,
     LockOutlined,
+    LspDebug,
+    LspRestart,
+    LspStop,
     MagnifyingGlass,
     MailOpen,
     Maximize,
@@ -207,6 +213,7 @@ pub enum IconName {
     Save,
     Scissors,
     Screen,
+    ScrollText,
     SearchCode,
     SearchSelection,
     SelectAll,
@@ -226,6 +233,7 @@ pub enum IconName {
     SparkleFilled,
     Spinner,
     Split,
+    SplitAlt,
     SquareDot,
     SquareMinus,
     SquarePlus,
@@ -264,6 +272,8 @@ pub enum IconName {
     ZedAssistantFilled,
     ZedBurnMode,
     ZedBurnModeOn,
+    ZedMcpCustom,
+    ZedMcpExtension,
     ZedPredict,
     ZedPredictDisabled,
     ZedPredictDown,
