@@ -6,32 +6,32 @@ pub use remote_video_track_view::{RemoteVideoTrackView, RemoteVideoTrackViewEven
 #[cfg(not(any(
     test,
     feature = "test-support",
-    all(target_os = "windows", target_env = "gnu")
+    any(all(target_os = "windows", target_env = "gnu"), target_os = "freebsd")
 )))]
 mod livekit_client;
 #[cfg(not(any(
     test,
     feature = "test-support",
-    all(target_os = "windows", target_env = "gnu")
+    any(all(target_os = "windows", target_env = "gnu"), target_os = "freebsd")
 )))]
 pub use livekit_client::*;
 
 #[cfg(any(
     test,
     feature = "test-support",
-    all(target_os = "windows", target_env = "gnu")
+    any(all(target_os = "windows", target_env = "gnu"), target_os = "freebsd")
 ))]
 mod mock_client;
 #[cfg(any(
     test,
     feature = "test-support",
-    all(target_os = "windows", target_env = "gnu")
+    any(all(target_os = "windows", target_env = "gnu"), target_os = "freebsd")
 ))]
 pub mod test;
 #[cfg(any(
     test,
     feature = "test-support",
-    all(target_os = "windows", target_env = "gnu")
+    any(all(target_os = "windows", target_env = "gnu"), target_os = "freebsd")
 ))]
 pub use mock_client::*;
 
