@@ -36,13 +36,8 @@ impl ContextServerStatus {
         match state {
             ContextServerState::Starting { .. } => ContextServerStatus::Starting,
             ContextServerState::Running { .. } => ContextServerStatus::Running,
-            ContextServerState::Stopped { error, .. } => {
-                if let Some(error) = error {
-                    ContextServerStatus::Error(error.clone())
-                } else {
-                    ContextServerStatus::Stopped
-                }
-            }
+            ContextServerState::Stopped { .. } => ContextServerStatus::Stopped,
+            ContextServerState::Error { error, .. } => ContextServerStatus::Error(error.clone()),
         }
     }
 }
@@ -60,7 +55,11 @@ enum ContextServerState {
     Stopped {
         server: Arc<ContextServer>,
         configuration: Arc<ContextServerConfiguration>,
-        error: Option<Arc<str>>,
+    },
+    Error {
+        server: Arc<ContextServer>,
+        configuration: Arc<ContextServerConfiguration>,
+        error: Arc<str>,
     },
 }
 
@@ -70,6 +69,7 @@ impl ContextServerState {
             ContextServerState::Starting { server, .. } => server.clone(),
             ContextServerState::Running { server, .. } => server.clone(),
             ContextServerState::Stopped { server, .. } => server.clone(),
+            ContextServerState::Error { server, .. } => server.clone(),
         }
     }
 
@@ -78,11 +78,12 @@ impl ContextServerState {
             ContextServerState::Starting { configuration, .. } => configuration.clone(),
             ContextServerState::Running { configuration, .. } => configuration.clone(),
             ContextServerState::Stopped { configuration, .. } => configuration.clone(),
+            ContextServerState::Error { configuration, .. } => configuration.clone(),
         }
     }
 }
 
-#[derive(PartialEq, Eq)]
+#[derive(Debug, PartialEq, Eq)]
 pub enum ContextServerConfiguration {
     Custom {
         command: ContextServerCommand,
@@ -109,10 +110,14 @@ impl ContextServerConfiguration {
         cx: &AsyncApp,
     ) -> Option<Self> {
         match settings {
-            ContextServerSettings::Custom { command } => {
-                Some(ContextServerConfiguration::Custom { command })
-            }
-            ContextServerSettings::Extension { settings } => {
+            ContextServerSettings::Custom {
+                enabled: _,
+                command,
+            } => Some(ContextServerConfiguration::Custom { command }),
+            ContextServerSettings::Extension {
+                enabled: _,
+                settings,
+            } => {
                 let descriptor = cx
                     .update(|cx| registry.read(cx).context_server_descriptor(&id.0))
                     .ok()
@@ -130,6 +135,7 @@ pub type ContextServerFactory =
     Box<dyn Fn(ContextServerId, Arc<ContextServerConfiguration>) -> Arc<ContextServer>>;
 
 pub struct ContextServerStore {
+    context_server_settings: HashMap<Arc<str>, ContextServerSettings>,
     servers: HashMap<ContextServerId, ContextServerState>,
     worktree_store: Entity<WorktreeStore>,
     registry: Entity<ContextServerDescriptorRegistry>,
@@ -197,6 +203,11 @@ impl ContextServerStore {
                     this.available_context_servers_changed(cx);
                 }),
                 cx.observe_global::<SettingsStore>(|this, cx| {
+                    let settings = Self::resolve_context_server_settings(&this.worktree_store, cx);
+                    if &this.context_server_settings == settings {
+                        return;
+                    }
+                    this.context_server_settings = settings.clone();
                     this.available_context_servers_changed(cx);
                 }),
             ]
@@ -206,6 +217,8 @@ impl ContextServerStore {
 
         let mut this = Self {
             _subscriptions: subscriptions,
+            context_server_settings: Self::resolve_context_server_settings(&worktree_store, cx)
+                .clone(),
             worktree_store,
             registry,
             needs_server_update: false,
@@ -235,6 +248,13 @@ impl ContextServerStore {
         self.servers.get(id).map(ContextServerStatus::from_state)
     }
 
+    pub fn configuration_for_server(
+        &self,
+        id: &ContextServerId,
+    ) -> Option<Arc<ContextServerConfiguration>> {
+        self.servers.get(id).map(|state| state.configuration())
+    }
+
     pub fn all_server_ids(&self) -> Vec<ContextServerId> {
         self.servers.keys().cloned().collect()
     }
@@ -256,15 +276,17 @@ impl ContextServerStore {
         cx.spawn(async move |this, cx| {
             let this = this.upgrade().context("Context server store dropped")?;
             let settings = this
-                .update(cx, |this, cx| {
-                    this.context_server_settings(cx)
-                        .get(&server.id().0)
-                        .cloned()
+                .update(cx, |this, _| {
+                    this.context_server_settings.get(&server.id().0).cloned()
                 })
                 .ok()
                 .flatten()
                 .context("Failed to get context server settings")?;
 
+            if !settings.enabled() {
+                return Ok(());
+            }
+
             let (registry, worktree_store) = this.update(cx, |this, _| {
                 (this.registry.clone(), this.worktree_store.clone())
             })?;
@@ -286,6 +308,13 @@ impl ContextServerStore {
     }
 
     pub fn stop_server(&mut self, id: &ContextServerId, cx: &mut Context<Self>) -> Result<()> {
+        if matches!(
+            self.servers.get(id),
+            Some(ContextServerState::Stopped { .. })
+        ) {
+            return Ok(());
+        }
+
         let state = self
             .servers
             .remove(id)
@@ -304,7 +333,6 @@ impl ContextServerStore {
             ContextServerState::Stopped {
                 configuration,
                 server,
-                error: None,
             },
             cx,
         );
@@ -364,10 +392,10 @@ impl ContextServerStore {
                         this.update(cx, |this, cx| {
                             this.update_server_state(
                                 id.clone(),
-                                ContextServerState::Stopped {
+                                ContextServerState::Error {
                                     configuration,
                                     server,
-                                    error: Some(err.to_string().into()),
+                                    error: err.to_string().into(),
                                 },
                                 cx,
                             )
@@ -417,12 +445,11 @@ impl ContextServerStore {
         }
     }
 
-    fn context_server_settings<'a>(
-        &'a self,
+    fn resolve_context_server_settings<'a>(
+        worktree_store: &'a Entity<WorktreeStore>,
         cx: &'a App,
     ) -> &'a HashMap<Arc<str>, ContextServerSettings> {
-        let location = self
-            .worktree_store
+        let location = worktree_store
             .read(cx)
             .visible_worktrees(cx)
             .next()
@@ -470,9 +497,9 @@ impl ContextServerStore {
     }
 
     async fn maintain_servers(this: WeakEntity<Self>, cx: &mut AsyncApp) -> Result<()> {
-        let (mut configured_servers, registry, worktree_store) = this.update(cx, |this, cx| {
+        let (mut configured_servers, registry, worktree_store) = this.update(cx, |this, _| {
             (
-                this.context_server_settings(cx).clone(),
+                this.context_server_settings.clone(),
                 this.registry.clone(),
                 this.worktree_store.clone(),
             )
@@ -483,12 +510,15 @@ impl ContextServerStore {
         {
             configured_servers
                 .entry(id)
-                .or_insert(ContextServerSettings::Extension {
-                    settings: serde_json::json!({}),
-                });
+                .or_insert(ContextServerSettings::default_extension());
         }
 
-        let configured_servers = join_all(configured_servers.into_iter().map(|(id, settings)| {
+        let (enabled_servers, disabled_servers): (HashMap<_, _>, HashMap<_, _>) =
+            configured_servers
+                .into_iter()
+                .partition(|(_, settings)| settings.enabled());
+
+        let configured_servers = join_all(enabled_servers.into_iter().map(|(id, settings)| {
             let id = ContextServerId(id);
             ContextServerConfiguration::from_settings(
                 settings,
@@ -513,13 +543,19 @@ impl ContextServerStore {
                 // All servers that are not in desired_servers should be removed from the store.
                 // This can happen if the user removed a server from the context server settings.
                 if !configured_servers.contains_key(&server_id) {
-                    servers_to_remove.insert(server_id.clone());
+                    if disabled_servers.contains_key(&server_id.0) {
+                        servers_to_stop.insert(server_id.clone());
+                    } else {
+                        servers_to_remove.insert(server_id.clone());
+                    }
                 }
             }
 
             for (id, config) in configured_servers {
-                let existing_config = this.servers.get(&id).map(|state| state.configuration());
-                if existing_config.as_deref() != Some(&config) {
+                let state = this.servers.get(&id);
+                let is_stopped = matches!(state, Some(ContextServerState::Stopped { .. }));
+                let existing_config = state.as_ref().map(|state| state.configuration());
+                if existing_config.as_deref() != Some(&config) || is_stopped {
                     let config = Arc::new(config);
                     if let Some(server) = this
                         .create_context_server(id.clone(), config.clone())
@@ -766,6 +802,7 @@ mod tests {
             vec![(
                 SERVER_1_ID.into(),
                 ContextServerSettings::Extension {
+                    enabled: true,
                     settings: json!({
                         "somevalue": true
                     }),
@@ -775,9 +812,9 @@ mod tests {
         .await;
 
         let executor = cx.executor();
-        let registry = cx.new(|_| {
+        let registry = cx.new(|cx| {
             let mut registry = ContextServerDescriptorRegistry::new();
-            registry.register_context_server_descriptor(SERVER_1_ID.into(), fake_descriptor_1);
+            registry.register_context_server_descriptor(SERVER_1_ID.into(), fake_descriptor_1, cx);
             registry
         });
         let store = cx.new(|cx| {
@@ -822,6 +859,7 @@ mod tests {
                 vec![(
                     server_1_id.0.clone(),
                     ContextServerSettings::Extension {
+                        enabled: true,
                         settings: json!({
                             "somevalue": false
                         }),
@@ -840,6 +878,7 @@ mod tests {
                 vec![(
                     server_1_id.0.clone(),
                     ContextServerSettings::Extension {
+                        enabled: true,
                         settings: json!({
                             "somevalue": false
                         }),
@@ -866,6 +905,7 @@ mod tests {
                     (
                         server_1_id.0.clone(),
                         ContextServerSettings::Extension {
+                            enabled: true,
                             settings: json!({
                                 "somevalue": false
                             }),
@@ -874,6 +914,7 @@ mod tests {
                     (
                         server_2_id.0.clone(),
                         ContextServerSettings::Custom {
+                            enabled: true,
                             command: ContextServerCommand {
                                 path: "somebinary".to_string(),
                                 args: vec!["arg".to_string()],
@@ -904,6 +945,7 @@ mod tests {
                     (
                         server_1_id.0.clone(),
                         ContextServerSettings::Extension {
+                            enabled: true,
                             settings: json!({
                                 "somevalue": false
                             }),
@@ -912,6 +954,7 @@ mod tests {
                     (
                         server_2_id.0.clone(),
                         ContextServerSettings::Custom {
+                            enabled: true,
                             command: ContextServerCommand {
                                 path: "somebinary".to_string(),
                                 args: vec!["anotherArg".to_string()],
@@ -937,6 +980,7 @@ mod tests {
                 vec![(
                     server_1_id.0.clone(),
                     ContextServerSettings::Extension {
+                        enabled: true,
                         settings: json!({
                             "somevalue": false
                         }),
@@ -951,6 +995,139 @@ mod tests {
                 assert_eq!(store.read(cx).status_for_server(&server_2_id), None);
             });
         }
+
+        // Ensure that nothing happens if the settings do not change
+        {
+            let _server_events = assert_server_events(&store, vec![], cx);
+            set_context_server_configuration(
+                vec![(
+                    server_1_id.0.clone(),
+                    ContextServerSettings::Extension {
+                        enabled: true,
+                        settings: json!({
+                            "somevalue": false
+                        }),
+                    },
+                )],
+                cx,
+            );
+
+            cx.run_until_parked();
+
+            cx.update(|cx| {
+                assert_eq!(
+                    store.read(cx).status_for_server(&server_1_id),
+                    Some(ContextServerStatus::Running)
+                );
+                assert_eq!(store.read(cx).status_for_server(&server_2_id), None);
+            });
+        }
+    }
+
+    #[gpui::test]
+    async fn test_context_server_enabled_disabled(cx: &mut TestAppContext) {
+        const SERVER_1_ID: &'static str = "mcp-1";
+
+        let server_1_id = ContextServerId(SERVER_1_ID.into());
+
+        let (_fs, project) = setup_context_server_test(
+            cx,
+            json!({"code.rs": ""}),
+            vec![(
+                SERVER_1_ID.into(),
+                ContextServerSettings::Custom {
+                    enabled: true,
+                    command: ContextServerCommand {
+                        path: "somebinary".to_string(),
+                        args: vec!["arg".to_string()],
+                        env: None,
+                    },
+                },
+            )],
+        )
+        .await;
+
+        let executor = cx.executor();
+        let registry = cx.new(|_| ContextServerDescriptorRegistry::new());
+        let store = cx.new(|cx| {
+            ContextServerStore::test_maintain_server_loop(
+                Box::new(move |id, _| {
+                    Arc::new(ContextServer::new(
+                        id.clone(),
+                        Arc::new(create_fake_transport(id.0.to_string(), executor.clone())),
+                    ))
+                }),
+                registry.clone(),
+                project.read(cx).worktree_store(),
+                cx,
+            )
+        });
+
+        // Ensure that mcp-1 starts up
+        {
+            let _server_events = assert_server_events(
+                &store,
+                vec![
+                    (server_1_id.clone(), ContextServerStatus::Starting),
+                    (server_1_id.clone(), ContextServerStatus::Running),
+                ],
+                cx,
+            );
+            cx.run_until_parked();
+        }
+
+        // Ensure that mcp-1 is stopped once it is disabled.
+        {
+            let _server_events = assert_server_events(
+                &store,
+                vec![(server_1_id.clone(), ContextServerStatus::Stopped)],
+                cx,
+            );
+            set_context_server_configuration(
+                vec![(
+                    server_1_id.0.clone(),
+                    ContextServerSettings::Custom {
+                        enabled: false,
+                        command: ContextServerCommand {
+                            path: "somebinary".to_string(),
+                            args: vec!["arg".to_string()],
+                            env: None,
+                        },
+                    },
+                )],
+                cx,
+            );
+
+            cx.run_until_parked();
+        }
+
+        // Ensure that mcp-1 is started once it is enabled again.
+        {
+            let _server_events = assert_server_events(
+                &store,
+                vec![
+                    (server_1_id.clone(), ContextServerStatus::Starting),
+                    (server_1_id.clone(), ContextServerStatus::Running),
+                ],
+                cx,
+            );
+            set_context_server_configuration(
+                vec![(
+                    server_1_id.0.clone(),
+                    ContextServerSettings::Custom {
+                        enabled: true,
+                        command: ContextServerCommand {
+                            path: "somebinary".to_string(),
+                            args: vec!["arg".to_string()],
+                            env: None,
+                        },
+                    },
+                )],
+                cx,
+            );
+
+            cx.run_until_parked();
+        }
     }
 
     fn set_context_server_configuration(
@@ -990,6 +1167,7 @@ mod tests {
 
     fn dummy_server_settings() -> ContextServerSettings {
         ContextServerSettings::Custom {
+            enabled: true,
             command: ContextServerCommand {
                 path: "somebinary".to_string(),
                 args: vec!["arg".to_string()],
