@@ -243,7 +243,7 @@ pub enum Request {
     Attach,
 }
 
-/// This struct represent a user created debug task from the new session modal
+/// This struct represent a user created debug task from the new process modal
 #[derive(Deserialize, Serialize, PartialEq, Eq, Clone, Debug, JsonSchema)]
 #[serde(rename_all = "snake_case")]
 pub struct ZedDebugConfig {
@@ -287,7 +287,8 @@ pub struct DebugTaskFile(pub Vec<DebugScenario>);
 
 impl DebugTaskFile {
     pub fn generate_json_schema(schemas: &AdapterSchemas) -> serde_json_lenient::Value {
-        let build_task_schema = schemars::schema_for!(BuildTaskDefinition);
+        let mut generator = schemars::generate::SchemaSettings::draft2019_09().into_generator();
+        let build_task_schema = generator.root_schema_for::<BuildTaskDefinition>();
         let mut build_task_value =
             serde_json_lenient::to_value(&build_task_schema).unwrap_or_default();
 
@@ -300,7 +301,12 @@ impl DebugTaskFile {
                 .get_mut("properties")
                 .and_then(|value| value.as_object_mut())
             {
-                properties.remove("label");
+                if properties.remove("label").is_none() {
+                    debug_panic!(
+                        "Generated TaskTemplate json schema did not have expected 'label' field. \
+                        Schema of 2nd alternative is: {template_object:?}"
+                    );
+                }
             }
 
             if let Some(arr) = template_object
@@ -310,13 +316,13 @@ impl DebugTaskFile {
                 arr.retain(|v| v.as_str() != Some("label"));
             }
         } else {
-            debug_panic!("Task Template schema in debug scenario's needs to be updated");
+            debug_panic!(
+                "Generated TaskTemplate json schema did not match expectations. \
+                Schema is: {build_task_value:?}"
+            );
         }
 
-        let task_definitions = build_task_value
-            .get("definitions")
-            .cloned()
-            .unwrap_or_default();
+        let task_definitions = build_task_value.get("$defs").cloned().unwrap_or_default();
 
         let adapter_conditions = schemas
             .0
@@ -374,7 +380,7 @@ impl DebugTaskFile {
                 },
                 "allOf": adapter_conditions
             },
-            "definitions": task_definitions
+            "$defs": task_definitions
         })
     }
 }
