@@ -44,7 +44,7 @@ pub struct SpawnInTerminal {
     /// Human readable name of the terminal tab.
     pub label: String,
     /// Executable command to spawn.
-    pub command: String,
+    pub command: Option<String>,
     /// Arguments to the command, potentially unsubstituted,
     /// to let the shell that spawns the command to do the substitution, if needed.
     pub args: Vec<String>,
@@ -387,20 +387,26 @@ impl ShellBuilder {
     }
 
     /// Returns the program and arguments to run this task in a shell.
-    pub fn build(mut self, task_command: String, task_args: &Vec<String>) -> (String, Vec<String>) {
-        let combined_command = task_args
-            .into_iter()
-            .fold(task_command, |mut command, arg| {
-                command.push(' ');
-                command.push_str(&arg);
-                command
-            });
-        self.args.extend(
-            self.interactive
-                .then(|| "-i".to_owned())
+    pub fn build(
+        mut self,
+        task_command: Option<String>,
+        task_args: &Vec<String>,
+    ) -> (String, Vec<String>) {
+        if let Some(task_command) = task_command {
+            let combined_command = task_args
                 .into_iter()
-                .chain(["-c".to_owned(), combined_command]),
-        );
+                .fold(task_command, |mut command, arg| {
+                    command.push(' ');
+                    command.push_str(&arg);
+                    command
+                });
+            self.args.extend(
+                self.interactive
+                    .then(|| "-i".to_owned())
+                    .into_iter()
+                    .chain(["-c".to_owned(), combined_command]),
+            );
+        }
 
         (self.program, self.args)
     }
@@ -428,21 +434,29 @@ impl ShellBuilder {
     }
 
     /// Returns the program and arguments to run this task in a shell.
-    pub fn build(mut self, task_command: String, task_args: &Vec<String>) -> (String, Vec<String>) {
-        let combined_command = task_args
-            .into_iter()
-            .fold(task_command, |mut command, arg| {
-                command.push(' ');
-                command.push_str(&self.to_windows_shell_variable(arg.to_string()));
-                command
-            });
+    pub fn build(
+        mut self,
+        task_command: Option<String>,
+        task_args: &Vec<String>,
+    ) -> (String, Vec<String>) {
+        if let Some(task_command) = task_command {
+            let combined_command = task_args
+                .into_iter()
+                .fold(task_command, |mut command, arg| {
+                    command.push(' ');
+                    command.push_str(&self.to_windows_shell_variable(arg.to_string()));
+                    command
+                });
 
-        match self.windows_shell_type() {
-            WindowsShellType::Powershell => self.args.extend(["-C".to_owned(), combined_command]),
-            WindowsShellType::Cmd => self.args.extend(["/C".to_owned(), combined_command]),
-            WindowsShellType::Other => {
-                self.args
-                    .extend(["-i".to_owned(), "-c".to_owned(), combined_command])
+            match self.windows_shell_type() {
+                WindowsShellType::Powershell => {
+                    self.args.extend(["-C".to_owned(), combined_command])
+                }
+                WindowsShellType::Cmd => self.args.extend(["/C".to_owned(), combined_command]),
+                WindowsShellType::Other => {
+                    self.args
+                        .extend(["-i".to_owned(), "-c".to_owned(), combined_command])
+                }
             }
         }
 
