@@ -2,14 +2,14 @@ use alacritty_terminal::vte::ansi::{
     CursorShape as AlacCursorShape, CursorStyle as AlacCursorStyle,
 };
 use collections::HashMap;
-use gpui::{
-    AbsoluteLength, App, FontFallbacks, FontFeatures, FontWeight, Pixels, SharedString, px,
-};
-use schemars::{JsonSchema, r#gen::SchemaGenerator, schema::RootSchema};
+use gpui::{AbsoluteLength, App, FontFallbacks, FontFeatures, FontWeight, Pixels, px};
+use schemars::JsonSchema;
 use serde_derive::{Deserialize, Serialize};
-use settings::{SettingsJsonSchemaParams, SettingsSources, add_references_to_properties};
+
+use settings::SettingsSources;
 use std::path::PathBuf;
 use task::Shell;
+use theme::FontFamilyName;
 
 #[derive(Copy, Clone, Debug, Serialize, Deserialize, JsonSchema, PartialEq, Eq)]
 #[serde(rename_all = "snake_case")]
@@ -29,7 +29,7 @@ pub struct TerminalSettings {
     pub shell: Shell,
     pub working_directory: WorkingDirectory,
     pub font_size: Option<Pixels>,
-    pub font_family: Option<SharedString>,
+    pub font_family: Option<FontFamilyName>,
     pub font_fallbacks: Option<FontFallbacks>,
     pub font_features: Option<FontFeatures>,
     pub font_weight: Option<FontWeight>,
@@ -147,13 +147,14 @@ pub struct TerminalSettingsContent {
     ///
     /// If this option is not included,
     /// the terminal will default to matching the buffer's font family.
-    pub font_family: Option<String>,
+    pub font_family: Option<FontFamilyName>,
 
     /// Sets the terminal's font fallbacks.
     ///
     /// If this option is not included,
     /// the terminal will default to matching the buffer's font fallbacks.
-    pub font_fallbacks: Option<Vec<String>>,
+    #[schemars(extend("uniqueItems" = true))]
+    pub font_fallbacks: Option<Vec<FontFamilyName>>,
 
     /// Sets the terminal's line height.
     ///
@@ -234,33 +235,13 @@ impl settings::Settings for TerminalSettings {
         sources.json_merge()
     }
 
-    fn json_schema(
-        generator: &mut SchemaGenerator,
-        params: &SettingsJsonSchemaParams,
-        _: &App,
-    ) -> RootSchema {
-        let mut root_schema = generator.root_schema_for::<Self::FileContent>();
-        root_schema.definitions.extend([
-            ("FontFamilies".into(), params.font_family_schema()),
-            ("FontFallbacks".into(), params.font_fallback_schema()),
-        ]);
-
-        add_references_to_properties(
-            &mut root_schema,
-            &[
-                ("font_family", "#/definitions/FontFamilies"),
-                ("font_fallbacks", "#/definitions/FontFallbacks"),
-            ],
-        );
-
-        root_schema
-    }
-
     fn import_from_vscode(vscode: &settings::VsCodeSettings, current: &mut Self::FileContent) {
         let name = |s| format!("terminal.integrated.{s}");
 
         vscode.f32_setting(&name("fontSize"), &mut current.font_size);
-        vscode.string_setting(&name("fontFamily"), &mut current.font_family);
+        if let Some(font_family) = vscode.read_string(&name("fontFamily")) {
+            current.font_family = Some(FontFamilyName(font_family.into()));
+        }
         vscode.bool_setting(&name("copyOnSelection"), &mut current.copy_on_select);
         vscode.bool_setting("macOptionIsMeta", &mut current.option_as_meta);
         vscode.usize_setting("scrollback", &mut current.max_scroll_history_lines);
