@@ -52,6 +52,7 @@ impl ThemeColors {
             element_active: neutral().light_alpha().step_5(),
             element_selected: neutral().light_alpha().step_5(),
             element_disabled: neutral().light_alpha().step_3(),
+            element_selection_background: blue().light().step_3().alpha(0.25),
             drop_target_background: blue().light_alpha().step_2(),
             ghost_element_background: system.transparent,
             ghost_element_hover: neutral().light_alpha().step_3(),
@@ -174,6 +175,7 @@ impl ThemeColors {
             element_active: neutral().dark_alpha().step_5(),
             element_selected: neutral().dark_alpha().step_5(),
             element_disabled: neutral().dark_alpha().step_3(),
+            element_selection_background: blue().dark().step_3().alpha(0.25),
             drop_target_background: blue().dark_alpha().step_2(),
             ghost_element_background: system.transparent,
             ghost_element_hover: neutral().dark_alpha().step_4(),
